#!/bin/sh /usr/share/centrifydc/perl/run
 
##############################################################################
# 
# Copyright (C) 2014-2017 Centrify Corporation. All rights reserved.
#
# This is an executable map file for automount.
# It is passed the mount point name and returns the map entry for it or
# returns nothing if it cannot find an entry.
#
# The map data comes from AD NIS map data.
#
# The map name is derived from the executable name, so the correct thing to
# do is symlink from /usr/share/centrifydc/etc/adauto.pl to /etc/auto.mymap
#
# Note that this script must run as root (or at least somebody with read
# rights to the krb5.keytab) because it uses our -m option on the ldapsearch
# command our ldap package must be installed
#
# This is a complete rewrite for 4.5:
# - use new Ldap library
# - store map in local DBM file
# - only reload it when the local file is too old
# - even then - avoid reload if we cant talk to a DC
# - load a config script
# - if run with no key value then reload - this is used to do asynch reload
#
# Note:
#     run with no key is required for the first time to build the database.
#
##############################################################################

use strict;
use lib '/usr/share/centrifydc/perl';
use CentrifyDC::Include;
use Getopt::Long;
use CentrifyDC::GP::CimsPath qw($CIMS_INSTALL);
Getopt::Long::Configure("bundling", "no_ignore_case");

use lib "../perl";
use File::Spec;
use File::Copy;
use File::Basename;
use Fcntl qw(:DEFAULT :flock);
use CentrifyDC::GP::General;

our $DBM_DIR = "/var/centrifydc/auto_maps";
our $DBM_DIR_TMP = "/var/centrifydc/auto_maps/tmp";
our $TIMESTAMP_KEY = "\$\$\$";
  
my $map = (File::Spec->splitpath($0))[2];
my $key = $ARGV[0]; 
my $LOCKFILE = "${DBM_DIR}/${map}.lck";

our %opts;
GetOptions( "g|debug" => \$opts{g},
            "h|help"  => \$opts{h} );
my $verbose;
$verbose = 1 if $opts{g};
if ($opts{h}) 
{
    do_usage();
    exit;
}

our $RELOAD_TIME = $CentrifyDC::Config::properties{"adauto.reloadtime"} || 60 * 30;
mkdir(${DBM_DIR}) unless stat(${DBM_DIR}); # just in case

our %map_dbm;
if (not $key) # we are calling ourselves to do async reload
{
   loadmap($map);
   exit;
}
# Check if the DBM exists or not
my $dbmexist = 0;
my $test_files = "${DBM_DIR}/${map}.* ${DBM_DIR}/${map}";
for my $mfile (glob $test_files) {
  if (-e $mfile) {
     $dbmexist = 1;
     last; 
  }
}
# if not, we shall load and create the DBM first.
if ($dbmexist == 0) {
    loadmap($map);
}
# lock DBM for reading first
my $FH;
open($FH, ">$LOCKFILE") or die "Cannot open lock file!";
flock($FH, LOCK_EX) or die "flock failed";
# Open DBM to read the key/value
dbmopen(%map_dbm, "${DBM_DIR}/${map}", 0666);

# First query the timestamp and the key/value
my $ts = $map_dbm{$TIMESTAMP_KEY};
my $v = $map_dbm{$key};

# Nothing is returned if the DBM has not been built yet.
$v = $map_dbm{"*"} unless $v;
print ${v}."\n" if $v;

# Now we should close the DBM.
dbmclose %map_dbm;
close($FH);
# Now check timestamp to see if we need to rebuild the DBM before exit.
if(not $ts or $verbose) {
   # reload if no timestamp (initial load) or if too old
   loadmap($map);
}
elsif (time - $ts > $RELOAD_TIME)
{
    # Fork a child to reload from AD, this is necessary especially when
    # there is a huge map defined on AD side.
    system "$0 &";
}

exit;

sub do_usage
{
    print <<EOF;
usage:
    Create a symbolic link with map name(eg: auto.host) under /etc to adauto.pl.
    Get map value: \t\t/etc/auto.host key
    Reload local cache map:\tadauto.pl  
    options:
         -g, --debug                print debug information
         -h, --help                 print this help information and exit
EOF
}

sub loadmap
{
    my $fname = "$CIMS_INSTALL/adedit/adlistnismaps";
    my @mapinfo = `$fname $map`;

    # If map is empty (non-existent or no entries)
    if (! @mapinfo)
    {
        exit;
    }

    my $FHEX;
    open($FHEX, ">$LOCKFILE") or die "Cannot open lock file for loading";
    flock($FHEX, LOCK_EX) or die "Cannot lock for loading";
    # The temp DBM file.
    our %tmp_dbm;
    mkdir(${DBM_DIR_TMP}) unless stat(${DBM_DIR_TMP}); # just in case

    dbmopen(%tmp_dbm, "${DBM_DIR_TMP}/${map}", 0666);

    $tmp_dbm{"\$\$\$"} = time;
    foreach my $line (@mapinfo)
    {
       chomp $line;
       $line =~ m/^(.*):\d+: (.*)$/;
       $tmp_dbm{$1} = $2;
       if ($verbose) {print "$1: $2\n";}
    }

    dbmclose %tmp_dbm;

    # Move the tmp DBM files (w/o the .dir, .pag extension) to replace the original DBM files.
    my $tmp_files = "${DBM_DIR_TMP}/${map}.* ${DBM_DIR_TMP}/${map}";
    for my $file (glob $tmp_files) {
       move($file, ${DBM_DIR});
    }
    close($FHEX);
}

